#!/usr/bin/env python
# -*- coding: utf-8 -*-

import openpyxl
import logging


class ExcelParser(object):
    def __init__(self):
        self.translations = {}

    def parse(self, excel_path):
        book = openpyxl.load_workbook(excel_path)
        sheet = book[book.sheetnames[0]]
        table = tuple(sheet.columns)

        for i in range(0, len(table)):
            if table[i][0].value in self.translations:
                continue
            if table[i][0].value is None:
                continue
            self.translations[table[i][0].value] = []
            for j in range(1, len(table[0])):
                string = table[i][j].value
                if string is None:
                    string = ""
                self.translations[table[i][0].value].append(string)


if __name__ == "__main__":
    excel_parser = ExcelParser()
    excel_parser.parse("tr.xlsx")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from opencc import OpenCC


class OpenccTranslator(object):
    def __init__(self, config):
        self.opencc = OpenCC(config)
        self.target_strings = []

    def generate(self, chs_strings):
        for string in chs_strings:
            self.target_strings.append(self.opencc.convert(string))
        return self.target_strings


if __name__ == "__main__":
    strings = ["软件", "对象", "中文"]
    translator = OpenccTranslator("s2hk")
    print(translator.generate(strings))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import xml.etree.ElementTree as ElementTree
import logging


class QtTs(object):
    def __init__(self, ts_path, tr_table):
        self.ts_path = ts_path
        self.xml_tree = ElementTree.parse(ts_path)
        self.tr_table = tr_table

    def __get_tr_list(self, locale_name):
        if locale_name in self.tr_table:
            return self.tr_table[locale_name]
        else:
            return []

    def __get_target_tr_list(self, locale_name):
        logging.info("target locale name: %s" % locale_name)
        tr_list = self.__get_tr_list(locale_name)
        if tr_list:
            return tr_list

        language_name = locale_name[0:2]
        for key in self.tr_table.keys():
            if key.startswith(language_name):
                logging.info("Found transltion %s" % key)
                return self.tr_table[key]

        logging.error("Fail to translate to %s", locale_name)
        raise KeyError("%s: Target translation NOT found" % locale_name)

    def __get_tr(self, source_string, source_tr_list, target_tr_list):
        try:
            index = source_tr_list.index(source_string)
            return target_tr_list[index]
        except (ValueError, IndexError):
            logging.error("no source string '%s' found" % source_string)
            raise

    def tr(self, locale_name):
        root = self.xml_tree.getroot()
        source_tr_list = self.__get_tr_list("source")
        if not source_tr_list:
            logging.error("no source list found")
            exit(-1)
        target_tr_list = self.__get_target_tr_list(locale_name)
        en_tr_list = self.__get_target_tr_list("en")
        for context in root:
            for message in context:
                if message.tag != "message":
                    continue
                source = message.find("source")
                target_string = self.__get_tr(source.text, source_tr_list,
                                              target_tr_list)
                if target_string is None:
                    continue
                if target_string.strip() == "":  #若翻译词条为空，使用英文翻译
                    target_string = self.__get_tr(source.text, source_tr_list,
                                                  en_tr_list)

                translation = message.find("translation")
                translation.attrib.clear()
                translation.text = target_string
                logging.info(
                    "translate %s to %s" % (source.text, translation.text))

    def save(self, path):
        self.xml_tree.write(path, encoding="utf-8", xml_declaration=True)
        with open(path, "r+", encoding="utf-8") as file:
            contents = file.readlines()
            contents.insert(1, "<!DOCTYPE TS>\n")
            file.seek(0)
            file.writelines(contents)


if __name__ == "__main__":
    ts = QtTs("./zh_CN.ts")
    ts.set_target_language("zh", "CN")
    ts.save()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='qtrer',
    version="1.0.0",
    description="对Qt翻译文件进行翻译的小工具",
    long_description="""根据翻译文档和OPENCC(简体和繁体转换)对Qt翻译文件进行翻译的小工具""",
    keywords='python Qt translate',
    author='xyz1001',
    author_email='zgzf1001@gmail.com',
    url=
    'https://github.com/xyz1001/PythonToolkit/tree/master/toolkit/QTranslatorHelper',
    license='MIT',
    py_modules=['qtrer', 'excel_parser', 'opencc_translator', 'qt_ts'],
    include_package_data=True,
    zip_safe=False,
    install_requires=['opencc-python-reimplemented', 'docopt', 'openpyxl'],
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
    entry_points={'console_scripts': [
        'qtrer = qtrer:main',
    ]},
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Translate Qt ts file

Usage:
    qtrer [--ts_dir=<ts_dir> --excel_dir=<excel_dir>]

Options:
    --ts_dir=<ts_dir>                       Qt翻译文件的目录
    --excel_dir=<excel_dir>               Excel翻译文件的路径
"""

import os
import logging
import sys
import docopt

from excel_parser import ExcelParser
from qt_ts import QtTs
from opencc_translator import OpenccTranslator


def main():
    arg = docopt.docopt(__doc__)
    qt_ts_file_dir = arg["--ts_dir"]
    translation_file_dir = arg["--excel_dir"]

    if qt_ts_file_dir is None:
        qt_ts_file_dir = "./translation"
    if translation_file_dir is None:
        translation_file_dir = "./doc/translation"

    translation = {}
    for item in os.listdir(translation_file_dir):
        file_path = os.path.join(translation_file_dir, item)
        if not os.path.isfile(file_path) or not file_path.endswith(".xlsx"):
            continue

        parser = ExcelParser()
        parser.parse(file_path)
        translation.update(parser.translations)

    zh_TW_translator = OpenccTranslator("s2twp")
    translation["zh_TW"] = zh_TW_translator.generate(translation["zh_CN"])

    zh_HK_translator = OpenccTranslator("s2hk")
    translation["zh_HK"] = zh_HK_translator.generate(translation["zh_CN"])
    logging.info(translation.keys())

    for item in os.listdir(qt_ts_file_dir):
        file_path = os.path.join(qt_ts_file_dir, item)
        if not file_path.endswith(".ts"):
            continue

        qt_ts = QtTs(file_path, translation)
        locale_name = os.path.splitext(os.path.basename(file_path))[0]
        try:
            qt_ts.tr(locale_name)
        except KeyError as e:
            logging.error(e.args)
            raise
        qt_ts.save(file_path)


if __name__ == "__main__":
    main()
